/*
** Copyright (C) 1999 Open Source Telecom Corporation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "answer.h"
#include <time.h>

void PK_CALLBACK portEvents(VPPORT *port, PK_U32 event, PK_U32 p1, PK_U32 p2)
{
	time_t now;
	struct tm *dt;
	int rc;

	switch(event)
	{
	case PK_EVENT_TIMER_EXPIRED:
		if(port->connected)
			syslog(LOG_INFO, "%s call duration expired", port->name);
	case PK_EVENT_DTMF_KEY_UP:
		if(!port->recording)
		{
			syslog(LOG_INFO, "stopping play on key");
			PK_AUDIO_Stop(port->hDSP);
			break;
		}
	case PK_EVENT_TRUNK_LC_TERM:
		PK_TIMER_Stop(port->hTIMER);
		if(!port->connected)
			break;

		PK_VP_DisableDTMFDetection(port->hDSP);
		syslog(LOG_INFO, "%s call disconnected", port->name);
		port->connected = PK_FALSE;
		PK_AUDIO_Stop(port->hDSP);
		PK_TRUNK_OnHook(port->hCTI);
	case PK_EVENT_DTMF_KEY_DOWN:
		if(port->recording)
			PK_AUDIO_Stop(port->hDSP);	
	case PK_EVENT_DTMF_ENERGY_LEVEL:
	case PK_EVENT_TRUNK_ONHOOK:
//		PK_VP_EnableToneDetection(port->hDSP); 
		break;
	case PK_EVENT_TRUNK_RINGING_ON:
		if(port->fd == -1)
		{
			time(&now);
			dt = localtime(&now);
			sprintf(port->recname, "/var/spool/vm/%02d%02d%02d%02d%02d%02d.ul",
				dt->tm_year, dt->tm_mon + 1, dt->tm_mday,
				dt->tm_hour, dt->tm_min, dt->tm_sec);
			syslog(LOG_DEBUG, "opening %s %d", port->recname, port->fd);
			port->fd = creat(port->recname, 0660);
		}
		break;

	case PK_EVENT_TGEN_TONE_PLAYED:
		if(!port->connected)
			break;

		port->recording = PK_TRUE;
		PK_TIMER_Start(port->hTIMER, MAX_RECORD * 1000 - 500);
		PK_VP_EnableDTMFDetection(port->hDSP);
		PK_AUDIO_InputAddBuffer(port->hDSP, &port->rbuf);
		PK_AUDIO_InputStart(port->hDSP);
		break;

	case PK_EVENT_TRUNK_RINGING_OFF:
		if(port->connected)
			break;
	
		port->connected = PK_TRUE;
		port->recording = PK_FALSE;
		port->pbuf.lpData = playbuf;
		port->pbuf.dwBufferLength = playsize;
		port->pbuf.dwBytesRecorded = playsize;
		port->pbuf.dwFlags = PK_AUDIO_LAST_BUFFER;
		port->pbuf.dwLoops = 0;
		port->pbuf.lpNext = 0;
		port->pbuf.reserved = 0;
		port->rbuf.lpData = port->rdata;
		port->rbuf.dwBufferLength = MAX_RECORD * 8000;
		port->rbuf.dwBytesRecorded = 0;
		port->rbuf.dwFlags = PK_AUDIO_LAST_BUFFER;
		port->rbuf.dwLoops = 0;
		port->rbuf.lpNext = 0;
		port->rbuf.reserved = 0;
		PK_TRUNK_OffHook(port->hCTI);
		break;

	case PK_EVENT_TRUNK_OFFHOOK:
		syslog(LOG_INFO, "%s call answered", port->name);
//		PK_VP_DisableToneDetection(port->hDSP);
		if(playsize)
		{
			PK_AUDIO_OutputSetSpeed(port->hDSP, PK_NORMAL_PLAYBACK_RATE);
			rc = PK_AUDIO_OutputAddBuffer(port->hDSP, &port->pbuf);
			rc += PK_AUDIO_OutputStart(port->hDSP);
			PK_VP_EnableDTMFDetection(port->hDSP);
			if(rc)
				syslog(LOG_INFO, "write failed %d", rc);
			else
				syslog(LOG_INFO, "playing %d", playsize);
		}	
		else
			PK_VP_PlayDTMFString(port->hDSP, "123");
		break;

	case PK_AUDIO_INPUT_IDLE:
		if(port->rbuf.dwBytesRecorded && port->fd > -1)
			write(port->fd, port->rdata, port->rbuf.dwBytesRecorded - 1000);
		close(port->fd);
		port->fd = -1;
		port->pid = fork();
		if(!port->pid)
		{
			pthread_kill_other_threads_np();
			port->pid = fork();
			if(port->pid)
				exit(0);
			execl("/usr/sbin/vmdeliver", "vmdeliver", port->recname, NULL);
			exit(-1);
		}	
		else
		{
			waitpid(port->pid, NULL, 0);
			port->pid = 0;
		}
	case PK_AUDIO_OUTPUT_IDLE:
		break;
	case PK_AUDIO_RETURN_BUFFER:
		if((PBufferHeader)p1 == &port->rbuf)
		{
			if(port->rbuf.dwBytesRecorded)
				syslog(LOG_DEBUG, "recorded %u bytes", port->rbuf.dwBytesRecorded);
		}
		else
		{
			PK_VP_DisableDTMFDetection(port->hDSP);
			PK_VP_PlayDTMFString(port->hDSP, "123");
		}
		break;
	default:
		syslog(LOG_DEBUG, "event %u %u %u", event, p1, p2);
	}
}

